#!/usr/bin/env python
# encoding: utf-8

"""
@file   : xkf_tonozip.py
@author : xukefang
@time   : 2022/5/26 10:18
"""

import os
import matplotlib

matplotlib.use('Agg')
import argparse
from PIL import Image
from models.experimental import attempt_load
from utils.torch_utils import select_device

import cv2
import numpy as np
import torch
import torch_mlu.core.mlu_quantize as mlu_quantize
import torch_mlu.core.mlu_model as ct
import torchvision.transforms as transforms

"""
请在训练环境完成版本降级,attempt_load,select_device都是原版接口
"""


def get_model(input_pt, input_yaml):
    from models.yolo import Model
    device = select_device('')
    # Create model
    model = Model(input_yaml).to(device)
    print("load : {}".format(input_pt))
    state_dict = torch.load(input_pt,map_location=torch.device(device))
    model.load_state_dict(state_dict, strict=False)
    return model


def quant(input_pt, input_yaml, out_pt):
    model = torch.jit.load(input_pt) if 'torchscript' in input_pt else get_model(input_pt,input_yaml)
    stride = int(model.stride.max())  # model stride
    print("stride = {}".format(stride))
    names = model.module.names if hasattr(model, 'module') else model.names  # get class names
    print("names = {}".format(names))
    net = model.eval()
    # 配置量化参数
    qconfig = {'iteration': 1, 'use_avg': False, 'data_scale': 1.0, 'firstconv': False, 'per_channel': False}
    # 调用量化接口
    quantized_net = mlu_quantize.quantize_dynamic_mlu(net.float(), qconfig_spec=qconfig, dtype='int8', gen_quant=True)
    # 设置为推理模式
    quantized_net = quantized_net.eval().float()
    # 对图片作预处理
    img_mat = Image.open("./data/images/bus.jpg")
    if img_mat.mode != 'RGB':
        img_mat = img_mat.convert('RGB')
    crop = 640
    resize = 640
    ct.set_core_version("MLU220")
    ct.set_device(-1)
    transform = transforms.Compose([
        transforms.Resize(resize),
        transforms.CenterCrop(crop),
        transforms.ToTensor(),
    ])
    img = transform(img_mat)
    im_tensor = torch.unsqueeze(img, 0)
    im_tensor = im_tensor.float()
    print(im_tensor.shape)
    # 执行推理生成量化值
    print("quantized_net======= ")
    quantized_net(im_tensor)
    print("quantized_net======= done")
    # 保存量化模型
    torch.save(quantized_net.state_dict(), out_pt)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("--input_pt", default="./yolov5s.v6-person-nozip.pt", required=True,
                        help="输入pt")
    parser.add_argument("--input_yaml", default="./models/yolov5s-person.yaml", required=True,
                        help="输入pt")
    parser.add_argument("--out_pt", default="./yolov5s.v6-person-int8.pt", required=False,
                        help="输出pt")
    args = parser.parse_args()
    input_pt = os.path.abspath(args.input_pt)
    input_yaml = os.path.abspath(args.input_yaml)
    out_pt = os.path.abspath(args.out_pt)
    quant(input_pt, input_yaml, out_pt)
