#!/usr/bin/env python
# encoding: utf-8

"""
@file   : cocolabel2person.py
@author : xukefang
@time   : 2021/12/8 14:32
"""

import os
import argparse


def glog_folder(path):
    fs = []
    if not os.path.exists(path): return fs
    for it in os.listdir(path):
        item = os.path.join(path, it)
        if os.path.isdir(item):
            fs.append(item)
    return fs


def glob_format(path, fmt_list=('.jpg', '.png', '.bmp'), base_name=False):
    fs = []
    if not os.path.exists(path): return fs
    for root, dirs, files in os.walk(path):
        for file in files:
            item = os.path.join(root, file)
            fmt = os.path.splitext(item)[-1]
            if fmt not in fmt_list: continue
            if base_name:
                fs.append(os.path.splitext(file)[0])
            else:
                fs.append(item)
    return fs


def labels2person(data_path, valid_labels):
    label_files = glob_format(data_path, fmt_list=(".txt",))
    for lfile in label_files:
        data = []
        with open(lfile,'r') as f:
            data = f.readlines()
        if not data:
            continue
        infos = list(data)
        person_infos = []
        for info in infos:
            if info[0] in valid_labels:
                person_infos.append(info)
        if not person_infos:
            os.remove(lfile)
            print("remove empty {}".format(lfile))
        else:
            with open(lfile,'w') as wf:
                wf.writelines(person_infos)



if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("data_path")
    parser.add_argument("-labels", "--valid_labels", type=list, default=["0"])
    args = parser.parse_args()
    data_path = os.path.abspath(args.data_path)
    valid_labels = args.valid_labels
    print("data_path:{}".format(data_path))
    print("valid_labels:{}".format(valid_labels))
    labels2person(data_path, valid_labels)
